use brass_config::{Config, SmtpTlsType};
use lettre::{
    transport::{
        smtp::{authentication::Credentials, extension::ClientId},
        stub::StubTransport,
    },
    SmtpTransport, Transport,
};

use crate::utils::ApplicationError;

mod forgot_password;
mod registration;

pub struct Mailer {
    transport: Transports,
    hostname: String,
}

enum Transports {
    SmtpTransport(SmtpTransport),
    StubTransport(StubTransport),
}

impl Transport for Transports {
    type Ok = ();
    type Error = ApplicationError;

    fn send_raw(
        &self,
        envelope: &lettre::address::Envelope,
        email: &[u8],
    ) -> Result<Self::Ok, Self::Error> {
        match self {
            Transports::SmtpTransport(smtp_transport) => smtp_transport
                .send_raw(envelope, email)
                .map(|_| ())
                .map_err(|err| ApplicationError::EmailTransport(err)),
            Transports::StubTransport(stub_transport) => stub_transport
                .send_raw(envelope, email)
                .map(|_| ())
                .map_err(|err| ApplicationError::EmailStubTransport(err)),
        }
    }
}

impl Mailer {
    pub fn new(config: &Config) -> anyhow::Result<Self> {
        let mut builder = match config.smtp_tlstype {
            SmtpTlsType::StartTLS => {
                SmtpTransport::starttls_relay(&config.smtp_server)?.port(config.smtp_port)
            }
            SmtpTlsType::TLS => SmtpTransport::relay(&config.smtp_server)?.port(config.smtp_port),
            SmtpTlsType::NoTLS => {
                SmtpTransport::builder_dangerous(&config.smtp_server).port(config.smtp_port)
            }
        };

        if let (Some(login), Some(password)) =
            (config.smtp_login.as_ref(), config.smtp_password.as_ref())
        {
            builder =
                builder.credentials(Credentials::new(login.to_string(), password.to_string()));
        }

        let transport = builder
            .hello_name(ClientId::Domain(config.hostname.clone()))
            .build();

        let mailer = Mailer {
            transport: Transports::SmtpTransport(transport),
            hostname: config.hostname.to_string(),
        };

        Ok(mailer)
    }
}
