use lettre::{
    message::{header::ContentType, Mailbox, MultiPart, SinglePart},
    Message, Transport,
};
use rinja::Template;

use crate::{models::User, utils::ApplicationError};

use super::Mailer;

impl Mailer {
    pub fn send_registration_mail(&self, user: &User, token: &str) -> Result<(), ApplicationError> {
        let message = build(&self.hostname, &user.name, &user.email, token)?;
        self.transport.send(&message)?;

        Ok(())
    }
}

#[derive(Template)]
#[template(path = "emails/register.txt")]
struct RegisterMailTemplatePlain<'a> {
    name: &'a str,
    hostname: &'a str,
    register_url: &'a str,
}

#[derive(Template)]
#[template(path = "emails/register.html")]
struct RegisterMailTemplateHtml<'a> {
    name: &'a str,
    hostname: &'a str,
    register_url: &'a str,
}

fn build(
    hostname: &str,
    name: &str,
    email: &str,
    token: &str,
) -> Result<Message, ApplicationError> {
    let register_url = format!("https://{hostname}/register?token={token}");

    let plain = RegisterMailTemplatePlain {
        name,
        hostname,
        register_url: &register_url,
    }
    .to_string();

    let html = RegisterMailTemplateHtml {
        name,
        hostname,
        register_url: &register_url,
    }
    .to_string();

    let message = Message::builder()
        .from("noreply <noreply@brasiwa-leipzig.de>".parse()?)
        .reply_to("noreply <noreply@brasiwa-leipzig.de>".parse()?)
        .to(Mailbox::new(Some(name.to_string()), email.parse()?))
        .subject("Brass: Registrierung deines Accounts")
        .multipart(
            MultiPart::alternative()
                .singlepart(
                    SinglePart::builder()
                        .header(ContentType::TEXT_PLAIN)
                        .body(plain),
                )
                .singlepart(
                    SinglePart::builder()
                        .header(ContentType::TEXT_HTML)
                        .body(html),
                ),
        )?;

    Ok(message)
}

#[test]
fn build_mail_snapshot() {
    let message = build(
        "brasiwa-leipzig.de",
        "Max Mustermann",
        "max@example.com",
        "123456789",
    )
    .unwrap();

    let sender = lettre::transport::stub::StubTransport::new_ok();
    sender.send(&message).unwrap();
    let messages = sender.messages();
    let (_, sent_message) = messages.first().unwrap();

    crate::utils::test_helper::assert_mail_snapshot!(sent_message);
}
